@@ -145,6 +145,19 @@ class WebView(QWebView):
         self.loadProgress.connect(lambda p: setattr(self, 'progress', p))
         objreg.get('config').changed.connect(self.on_config_changed)
 
+    @pyqtSlot()
+    def on_initial_layout_completed(self):
+        """Add url to history now that we have displayed something."""
+        history = objreg.get('web-history')
+        no_formatting = QUrl.UrlFormattingOption(0)
+        orig_url = self.page().mainFrame().requestedUrl()
+        if (orig_url.isValid() and
+                not orig_url.matches(self.cur_url, no_formatting)):
+            # If the url of the page is different than the url of the link
+            # originally clicked, save them both.
+            history.add_url(orig_url, self.title(), redirect=True)
+        history.add_url(self.cur_url, self.title())
+
     def _init_page(self):
         """Initialize the QWebPage used by this view."""
         page = webpage.BrowserPage(self.win_id, self.tab_id, self)
@@ -152,6 +165,8 @@ class WebView(QWebView):
         page.linkHovered.connect(self.linkHovered)
         page.mainFrame().loadStarted.connect(self.on_load_started)
         page.mainFrame().loadFinished.connect(self.on_load_finished)
+        page.mainFrame().initialLayoutCompleted.connect(
+            self.on_initial_layout_completed)
         page.statusBarMessage.connect(
             lambda msg: setattr(self, 'statusbar_message', msg))
         page.networkAccessManager().sslErrors.connect(
@@ -213,7 +228,7 @@ class WebView(QWebView):
         """Initialize the _zoom neighborlist."""
         levels = config.get('ui', 'zoom-levels')
         self._zoom = usertypes.NeighborList(
-            levels, mode=usertypes.NeighborList.Modes.block)
+            levels, mode=usertypes.NeighborList.Modes.edge)
         self._zoom.fuzzyval = config.get('ui', 'default-zoom')
 
     def _mousepress_backforward(self, e):
@@ -357,7 +372,8 @@ class WebView(QWebView):
         """Add the javascript bridge for qute:... pages."""
         frame = self.sender()
         if not isinstance(frame, QWebFrame):
-            log.webview.error("Got non-QWebFrame in add_js_bridge")
+            log.webview.error("Got non-QWebFrame {!r} in "
+                              "add_js_bridge!".format(frame))
             return
 
         if frame.url().scheme() == 'qute':
