@@ -71,6 +71,8 @@ class _QtFIFOReader(QObject):
     def cleanup(self):
         """Clean up so the FIFO can be closed."""
         self._notifier.setEnabled(False)
+        for line in self._fifo:
+            self.got_line.emit(line.rstrip('\r\n'))
         self._fifo.close()
 
 
@@ -82,6 +84,7 @@ class _BaseUserscriptRunner(QObject):
         _filepath: The path of the file/FIFO which is being read.
         _proc: The GUIProcess which is being executed.
         _win_id: The window ID this runner is associated with.
+        _cleaned_up: Whether temporary files were cleaned up.
 
     Signals:
         got_cmd: Emitted when a new command arrived and should be executed.
@@ -93,6 +96,7 @@ class _BaseUserscriptRunner(QObject):
 
     def __init__(self, win_id, parent=None):
         super().__init__(parent)
+        self._cleaned_up = False
         self._win_id = win_id
         self._filepath = None
         self._proc = None
@@ -114,10 +118,14 @@ class _BaseUserscriptRunner(QObject):
                                            additional_env=self._env,
                                            verbose=verbose, parent=self)
         self._proc.finished.connect(self.on_proc_finished)
+        self._proc.error.connect(self.on_proc_error)
         self._proc.start(cmd, args)
 
     def _cleanup(self):
         """Clean up temporary files."""
+        if self._cleaned_up:
+            return
+        self._cleaned_up = True
         tempfiles = [self._filepath]
         if 'QUTE_HTML' in self._env:
             tempfiles.append(self._env['QUTE_HTML'])
@@ -150,6 +158,7 @@ class _BaseUserscriptRunner(QObject):
         """
         raise NotImplementedError
 
+    @pyqtSlot()
     def on_proc_finished(self):
         """Called when the process has finished.
 
@@ -157,6 +166,14 @@ class _BaseUserscriptRunner(QObject):
         """
         raise NotImplementedError
 
+    @pyqtSlot()
+    def on_proc_error(self):
+        """Called when the process encountered an error.
+
+        Needs to be overridden by subclasses.
+        """
+        raise NotImplementedError
+
 
 class _POSIXUserscriptRunner(_BaseUserscriptRunner):
 
@@ -184,8 +201,8 @@ class _POSIXUserscriptRunner(_BaseUserscriptRunner):
             # pylint: disable=no-member,useless-suppression
             os.mkfifo(self._filepath)
         except OSError as e:
-            message.error(self._win_id, "Error while creating FIFO: {}".format(
-                e))
+            message.error(self._win_id,
+                          "Error while creating FIFO: {}".format(e))
             return
 
         self._reader = _QtFIFOReader(self._filepath)
@@ -193,12 +210,18 @@ class _POSIXUserscriptRunner(_BaseUserscriptRunner):
 
         self._run_process(cmd, *args, env=env, verbose=verbose)
 
+    @pyqtSlot()
     def on_proc_finished(self):
-        """Interrupt the reader when the process finished."""
-        self.finish()
+        self._cleanup()
 
-    def finish(self):
-        """Quit the thread and clean up when the reader finished."""
+    @pyqtSlot()
+    def on_proc_error(self):
+        self._cleanup()
+
+    def _cleanup(self):
+        """Clean up reader and temporary files."""
+        if self._cleaned_up:
+            return
         log.procs.debug("Cleaning up")
         self._reader.cleanup()
         self._reader.deleteLater()
@@ -229,24 +252,33 @@ class _WindowsUserscriptRunner(_BaseUserscriptRunner):
 
     def _cleanup(self):
         """Clean up temporary files after the userscript finished."""
-        try:
-            os.close(self._oshandle)
-        except OSError:
-            log.procs.exception("Failed to close file handle!")
-        super()._cleanup()
-        self._oshandle = None
+        if self._cleaned_up:
+            return
 
-    def on_proc_finished(self):
-        """Read back the commands when the process finished."""
         try:
             with open(self._filepath, 'r', encoding='utf-8') as f:
                 for line in f:
                     self.got_cmd.emit(line.rstrip())
         except OSError:
             log.procs.exception("Failed to read command file!")
-        self._cleanup()
+
+        try:
+            os.close(self._oshandle)
+        except OSError:
+            log.procs.exception("Failed to close file handle!")
+        super()._cleanup()
+        self._oshandle = None
         self.finished.emit()
 
+    @pyqtSlot()
+    def on_proc_error(self):
+        self._cleanup()
+
+    @pyqtSlot()
+    def on_proc_finished(self):
+        """Read back the commands when the process finished."""
+        self._cleanup()
+
     def run(self, cmd, *args, env=None, verbose=False):
         try:
             self._oshandle, self._filepath = tempfile.mkstemp(text=True)
@@ -335,11 +367,11 @@ def run(cmd, *args, win_id, env, verbose=False):
     """
     tabbed_browser = objreg.get('tabbed-browser', scope='window',
                                 window=win_id)
-    commandrunner = runners.CommandRunner(win_id, tabbed_browser)
+    commandrunner = runners.CommandRunner(win_id, parent=tabbed_browser)
     runner = UserscriptRunner(win_id, tabbed_browser)
     runner.got_cmd.connect(
-        lambda cmd: log.commands.debug("Got userscript command: {}".format(
-            cmd)))
+        lambda cmd:
+        log.commands.debug("Got userscript command: {}".format(cmd)))
     runner.got_cmd.connect(commandrunner.run_safely)
     user_agent = config.get('network', 'user-agent')
     if user_agent is not None:
