@@ -29,7 +29,8 @@ import functools
 
 from PyQt5.QtCore import pyqtSlot
 
-from qutebrowser.completion.models import miscmodels, urlmodel, configmodel
+from qutebrowser.completion.models import (miscmodels, urlmodel, configmodel,
+                                           base)
 from qutebrowser.utils import objreg, usertypes, log, debug
 from qutebrowser.config import configdata
 
@@ -119,6 +120,13 @@ def init_session_completion():
     _instances[usertypes.Completion.sessions] = model
 
 
+def _init_empty_completion():
+    """Initialize empty completion model."""
+    log.completion.debug("Initializing empty completion.")
+    if usertypes.Completion.empty not in _instances:
+        _instances[usertypes.Completion.empty] = base.BaseCompletionModel()
+
+
 INITIALIZERS = {
     usertypes.Completion.command: _init_command_completion,
     usertypes.Completion.helptopic: _init_helptopic_completion,
@@ -130,6 +138,7 @@ INITIALIZERS = {
     usertypes.Completion.quickmark_by_name: init_quickmark_completions,
     usertypes.Completion.bookmark_by_url: init_bookmark_completions,
     usertypes.Completion.sessions: init_session_completion,
+    usertypes.Completion.empty: _init_empty_completion,
 }
 
 
@@ -177,3 +186,7 @@ def init():
     history = objreg.get('web-history')
     history.async_read_done.connect(
         functools.partial(update, [usertypes.Completion.url]))
+
+    keyconf = objreg.get('key-config')
+    keyconf.changed.connect(
+        functools.partial(update, [usertypes.Completion.command]))
