@@ -341,6 +341,7 @@ class ConfigManager(QObject):
         ('colors', 'tab.indicator.system'): 'tabs.indicator.system',
         ('completion', 'history-length'): 'cmd-history-max-items',
         ('colors', 'downloads.fg'): 'downloads.fg.start',
+        ('ui', 'show-keyhints'): 'keyhint-blacklist',
     }
     DELETED_OPTIONS = [
         ('colors', 'tab.separator'),
@@ -360,6 +361,8 @@ class ConfigManager(QObject):
             _get_value_transformer({'false': '-1', 'true': '1000'}),
         ('general', 'log-javascript-console'):
             _get_value_transformer({'false': 'none', 'true': 'debug'}),
+        ('ui', 'keyhint-blacklist'):
+            _get_value_transformer({'false': '*', 'true': ''}),
     }
 
     changed = pyqtSignal(str, str)
@@ -686,9 +689,11 @@ class ConfigManager(QObject):
             raise cmdexc.CommandError("set: {} - {}".format(
                 e.__class__.__name__, e))
 
-    @cmdutils.register(name='set', instance='config', win_id='win_id',
-                       completion=[Completion.section, Completion.option,
-                                   Completion.value])
+    @cmdutils.register(name='set', instance='config')
+    @cmdutils.argument('section_', completion=Completion.section)
+    @cmdutils.argument('option', completion=Completion.option)
+    @cmdutils.argument('value', completion=Completion.value)
+    @cmdutils.argument('win_id', win_id=True)
     def set_command(self, win_id, section_=None, option=None, value=None,
                     temp=False, print_=False):
         """Set an option.
@@ -729,7 +734,7 @@ class ConfigManager(QObject):
                     val = self.get(section_, option)
                     layer = 'temp' if temp else 'conf'
                     if isinstance(val, bool):
-                        self.set(layer, section_, option, str(not val))
+                        self.set(layer, section_, option, str(not val).lower())
                     else:
                         raise cmdexc.CommandError(
                             "set: Attempted inversion of non-boolean value.")
