@@ -355,6 +355,12 @@ def data(readonly=False):
              "Hide the window decoration when using wayland "
              "(requires restart)"),
 
+            ('keyhint-blacklist',
+             SettingValue(typ.List(none_ok=True), ''),
+             "Keychains that shouldn't be shown in the keyhint dialog\n\n"
+             "Globs are supported, so ';*' will blacklist all keychains"
+             "starting with ';'. Use '*' to disable keyhints"),
+
             readonly=readonly
         )),
 
@@ -401,6 +407,10 @@ def data(readonly=False):
              SettingValue(typ.Bool(), 'true'),
              "Whether to try to pre-fetch DNS entries to speed up browsing."),
 
+            ('custom-headers',
+             SettingValue(typ.HeaderDict(none_ok=True), ''),
+             "Set custom headers for qutebrowser HTTP requests."),
+
             readonly=readonly
         )),
 
@@ -466,11 +476,15 @@ def data(readonly=False):
         ('input', sect.KeyValue(
             ('timeout',
              SettingValue(typ.Int(minval=0, maxval=MAXVALS['int']), '500'),
-             "Timeout for ambiguous key bindings."),
+             "Timeout for ambiguous key bindings.\n\n"
+             "If the current input forms both a complete match and a partial "
+             "match, the complete match will be executed after this time."),
 
             ('partial-timeout',
-             SettingValue(typ.Int(minval=0, maxval=MAXVALS['int']), '1000'),
-             "Timeout for partially typed key bindings."),
+             SettingValue(typ.Int(minval=0, maxval=MAXVALS['int']), '5000'),
+             "Timeout for partially typed key bindings.\n\n"
+             "If the current input forms only partial matches, the keystring "
+             "will be cleared after this time."),
 
             ('insert-mode-on-plugins',
              SettingValue(typ.Bool(), 'false'),
@@ -741,7 +755,7 @@ def data(readonly=False):
              "are not affected by this setting."),
 
             ('webgl',
-             SettingValue(typ.Bool(), 'true'),
+             SettingValue(typ.Bool(), 'false'),
              "Enables or disables WebGL."),
 
             ('css-regions',
@@ -884,7 +898,7 @@ def data(readonly=False):
             ('scatter',
              SettingValue(typ.Bool(), 'true'),
              "Whether to scatter hint key chains (like Vimium) or not (like "
-             "dwb)."),
+             "dwb). Ignored for number hints."),
 
             ('uppercase',
              SettingValue(typ.Bool(), 'false'),
@@ -899,6 +913,11 @@ def data(readonly=False):
              "Follow a hint immediately when the hint text is completely "
              "matched."),
 
+            ('auto-follow-timeout',
+             SettingValue(typ.Int(), '0'),
+             "A timeout to inhibit normal-mode key bindings after a successful"
+             "auto-follow."),
+
             ('next-regexes',
              SettingValue(typ.RegexList(flags=re.IGNORECASE),
                           r'\bnext\b,\bmore\b,\bnewer\b,\b[>→≫]\b,\b(>>|»)\b,'
@@ -911,6 +930,14 @@ def data(readonly=False):
                           r'\b(<<|«)\b'),
              "A comma-separated list of regexes to use for 'prev' links."),
 
+            ('find-implementation',
+             SettingValue(typ.String(
+                 valid_values=typ.ValidValues(
+                     ('javascript', "Better but slower"),
+                     ('python', "Slightly worse but faster"),
+                 )), 'javascript'),
+             "Which implementation to use to find elements to hint."),
+
             readonly=readonly
         )),
 
@@ -1192,6 +1219,18 @@ def data(readonly=False):
              "Background color for webpages if unset (or empty to use the "
              "theme's color)"),
 
+            ('keyhint.fg',
+             SettingValue(typ.QssColor(), '#FFFFFF'),
+             "Text color for the keyhint widget."),
+
+            ('keyhint.fg.suffix',
+             SettingValue(typ.CssColor(), '#FFFF00'),
+             "Highlight color for keys to complete the current keychain"),
+
+            ('keyhint.bg',
+             SettingValue(typ.QssColor(), 'rgba(0, 0, 0, 80%)'),
+             "Background color of the keyhint widget."),
+
             readonly=readonly
         )),
 
@@ -1273,6 +1312,10 @@ def data(readonly=False):
                  typ.Int(none_ok=True, minval=1, maxval=MAXVALS['int']), ''),
              "The default font size for fixed-pitch text."),
 
+            ('keyhint',
+             SettingValue(typ.Font(), DEFAULT_FONT_SIZE + ' ${_monospace}'),
+             "Font used in the keyhint widget."),
+
             readonly=readonly
         )),
     ])
@@ -1382,14 +1425,15 @@ KEY_DATA = collections.OrderedDict([
     ('normal', collections.OrderedDict([
         ('clear-keychain ;; search', ['<Escape>']),
         ('set-cmd-text -s :open', ['o']),
-        ('set-cmd-text :open {url}', ['go']),
+        ('set-cmd-text :open {url:pretty}', ['go']),
         ('set-cmd-text -s :open -t', ['O']),
-        ('set-cmd-text :open -t {url}', ['gO']),
+        ('set-cmd-text :open -t {url:pretty}', ['gO']),
         ('set-cmd-text -s :open -b', ['xo']),
-        ('set-cmd-text :open -b {url}', ['xO']),
+        ('set-cmd-text :open -b {url:pretty}', ['xO']),
         ('set-cmd-text -s :open -w', ['wo']),
-        ('set-cmd-text :open -w {url}', ['wO']),
+        ('set-cmd-text :open -w {url:pretty}', ['wO']),
         ('open -t', ['ga', '<Ctrl-T>']),
+        ('open -w', ['<Ctrl-N>']),
         ('tab-close', ['d', '<Ctrl-W>']),
         ('tab-close -o', ['D']),
         ('tab-only', ['co']),
@@ -1397,11 +1441,11 @@ KEY_DATA = collections.OrderedDict([
         ('tab-move', ['gm']),
         ('tab-move -', ['gl']),
         ('tab-move +', ['gr']),
-        ('tab-focus', ['J']),
-        ('tab-prev', ['K']),
+        ('tab-focus', ['J', '<Ctrl-PgDown>']),
+        ('tab-prev', ['K', '<Ctrl-PgUp>']),
         ('tab-clone', ['gC']),
-        ('reload', ['r']),
-        ('reload -f', ['R']),
+        ('reload', ['r', '<F5>']),
+        ('reload -f', ['R', '<Ctrl-F5>']),
         ('back', ['H']),
         ('back -t', ['th']),
         ('back -w', ['wh']),
@@ -1437,12 +1481,16 @@ KEY_DATA = collections.OrderedDict([
         ('search-prev', ['N']),
         ('enter-mode insert', ['i']),
         ('enter-mode caret', ['v']),
+        ('enter-mode set_mark', ['`']),
+        ('enter-mode jump_mark', ["'"]),
         ('yank', ['yy']),
         ('yank -s', ['yY']),
         ('yank -t', ['yt']),
         ('yank -ts', ['yT']),
         ('yank -d', ['yd']),
         ('yank -ds', ['yD']),
+        ('yank -p', ['yp']),
+        ('yank -ps', ['yP']),
         ('paste', ['pp']),
         ('paste -s', ['pP']),
         ('paste -t', ['Pp']),
