@@ -27,7 +27,7 @@ from PyQt5.QtCore import pyqtSignal, QObject
 
 from qutebrowser.config import configdata, textwrapper
 from qutebrowser.commands import cmdutils, cmdexc
-from qutebrowser.utils import log, utils, qtutils
+from qutebrowser.utils import log, utils, qtutils, message, usertypes
 
 
 class KeyConfigError(Exception):
@@ -151,18 +151,34 @@ class KeyConfigParser(QObject):
             f.write(data)
 
     @cmdutils.register(instance='key-config', maxsplit=1, no_cmd_split=True)
-    def bind(self, key, command, *, mode=None, force=False):
+    @cmdutils.argument('win_id', win_id=True)
+    @cmdutils.argument('key', completion=usertypes.Completion.empty)
+    @cmdutils.argument('command', completion=usertypes.Completion.command)
+    def bind(self, key, win_id, command=None, *, mode='normal', force=False):
         """Bind a key to a command.
 
         Args:
             key: The keychain or special key (inside `<...>`) to bind.
-            command: The command to execute, with optional args.
+            command: The command to execute, with optional args, or None to
+                     print the current binding.
             mode: A comma-separated list of modes to bind the key in
                   (default: `normal`).
             force: Rebind the key if it is already bound.
         """
-        if mode is None:
-            mode = 'normal'
+        if utils.is_special_key(key):
+            # <Ctrl-t>, <ctrl-T>, and <ctrl-t> should be considered equivalent
+            key = key.lower()
+
+        if command is None:
+            cmd = self.get_bindings_for(mode).get(key, None)
+            if cmd is None:
+                message.info(win_id, "{} is unbound in {} mode".format(
+                    key, mode))
+            else:
+                message.info(win_id, "{} is bound to '{}' in {} mode".format(
+                    key, cmd, mode))
+            return
+
         mode = self._normalize_sectname(mode)
         for m in mode.split(','):
             if m not in configdata.KEY_DATA:
@@ -183,7 +199,7 @@ class KeyConfigParser(QObject):
             self._mark_config_dirty()
 
     @cmdutils.register(instance='key-config')
-    def unbind(self, key, mode=None):
+    def unbind(self, key, mode='normal'):
         """Unbind a keychain.
 
         Args:
@@ -191,8 +207,10 @@ class KeyConfigParser(QObject):
             mode: A comma-separated list of modes to unbind the key in
                   (default: `normal`).
         """
-        if mode is None:
-            mode = 'normal'
+        if utils.is_special_key(key):
+            # <Ctrl-t>, <ctrl-T>, and <ctrl-t> should be considered equivalent
+            key = key.lower()
+
         mode = self._normalize_sectname(mode)
         for m in mode.split(','):
             if m not in configdata.KEY_DATA:
@@ -362,6 +380,9 @@ class KeyConfigParser(QObject):
 
     def _add_binding(self, sectname, keychain, command, *, force=False):
         """Add a new binding from keychain to command in section sectname."""
+        if utils.is_special_key(keychain):
+            # <Ctrl-t>, <ctrl-T>, and <ctrl-t> should be considered equivalent
+            keychain = keychain.lower()
         log.keyboard.vdebug("Adding binding {} -> {} in mode {}.".format(
             keychain, command, sectname))
         if sectname not in self.keybindings:
