@@ -123,7 +123,8 @@ def init_faulthandler(fileobj=sys.__stderr__):
         # start.
         return
     faulthandler.enable(fileobj)
-    if hasattr(faulthandler, 'register') and hasattr(signal, 'SIGUSR1'):
+    if (hasattr(faulthandler, 'register') and hasattr(signal, 'SIGUSR1') and
+            sys.stderr is not None):
         # If available, we also want a traceback on SIGUSR1.
         # pylint: disable=no-member,useless-suppression
         faulthandler.register(signal.SIGUSR1)
@@ -267,7 +268,8 @@ def remove_inputhook():
     """Remove the PyQt input hook.
 
     Doing this means we can't use the interactive shell anymore (which we don't
-    anyways), but we can use pdb instead."""
+    anyways), but we can use pdb instead.
+    """
     from PyQt5.QtCore import pyqtRemoveInputHook
     pyqtRemoveInputHook()
 
