@@ -170,7 +170,7 @@ class IPCServer(QObject):
         self._timer.setInterval(READ_TIMEOUT)
         self._timer.timeout.connect(self.on_timeout)
 
-        if os.name == 'nt':  # pragma: no coverage
+        if os.name == 'nt':  # pragma: no cover
             self._atime_timer = None
         else:
             self._atime_timer = usertypes.Timer(self, 'ipc-atime')
@@ -219,7 +219,13 @@ class IPCServer(QObject):
             # https://bugreports.qt.io/browse/QTBUG-48635
             #
             # This means we only use setSocketOption on Windows...
-            os.chmod(self._server.fullServerName(), 0o700)
+            try:
+                os.chmod(self._server.fullServerName(), 0o700)
+            except FileNotFoundError:
+                # https://github.com/The-Compiler/qutebrowser/issues/1530
+                # The server doesn't actually exist even if ok was reported as
+                # True, so report this as an error.
+                raise ListenError(self._server)
 
     @pyqtSlot('QLocalSocket::LocalSocketError')
     def on_error(self, err):
@@ -293,8 +299,7 @@ class IPCServer(QObject):
         try:
             decoded = data.decode('utf-8')
         except UnicodeDecodeError:
-            log.ipc.error("invalid utf-8: {}".format(
-                binascii.hexlify(data)))
+            log.ipc.error("invalid utf-8: {}".format(binascii.hexlify(data)))
             self._handle_invalid_data()
             return
 
@@ -455,8 +460,7 @@ def send_to_running_instance(socketname, command, target_arg, *,
     if socket is None:
         socket = QLocalSocket()
 
-    if (legacy_name is not None and
-            _has_legacy_server(legacy_name)):
+    if legacy_name is not None and _has_legacy_server(legacy_name):
         name_to_use = legacy_name
     else:
         name_to_use = socketname
